<?php

namespace Courseware\BlockTypes;

/**
 * This class represents the content of a Courseware chart block.
 *
 * @author  Ron Lucke <lucke@elan-ev.de>
 * @license GPL2 or any later version
 *
 * @since   Stud.IP 5.0
 */
class Chart extends BlockType
{
    public static function getType(): string
    {
        return 'chart';
    }

    public static function getTitle(): string
    {
        return _('Diagramm');
    }

    public static function getDescription(): string
    {
        return _('Präsentiert Datensätze in einem Diagramm.');
    }

    public function initialPayload(): array
    {
        return [
            'content' => [['label' => '', 'value' => '0', 'color' => 'blue']],
            'label' => '',
            'type' => 'bar',
        ];
    }

    public static function getJsonSchema(): string
    {
        $schemaFile = __DIR__.'/Chart.json';

        return file_get_contents($schemaFile);
    }

    public static function getCategories(): array
    {
        return ['multimedia', 'layout'];
    }

    public static function getContentTypes(): array
    {
        return ['data'];
    }

    public static function getFileTypes(): array
    {
        return [];
    }

    public static function getTags(): array
    {
        return [
            _('Darstellung'), _('Zahl'), _('Daten'), _('Säulendiagramm'), _('Balkendiagramm'),
            _('Kreisdiagramm'), _('Ringdiagramm'), _('Polardiagramm'), _('Liniendiagramm'),
            _('Visualisierung'), _('Informationsvisualisierung'), _('Datenvisualisierung'),
            _('deskriptiv'), _('Statistik'), _('Datensatz'), _('Skala'), _('Illustration'),
            _('Dekoration'), _('Datenpunkt'), _('Veranschaulichung'), _('Verlauf'), _('Zeitreihe'),
            _('Entwicklung'), _('Korrelation'), _('Diagramm')
        ];
    }
}
